/*****************************************************************************
 Cluster tiles described by textures

 IMPLEMENTATION

 Author: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>
 Copyright (c) 2006 Vision Research Lab, UCSB <http://vision.ece.ucsb.edu>

 History:
   04/19/2006 16:20 - First creation

 Ver : 1
*****************************************************************************/

#include <iostream>
#include <fstream>
#include <iostream>
#include <ostream>

#include <xvectors.h>

#include "clusterTextures.h"
#include "textureDescriptors.h"
#include "textureDescriptorsTxt.h"

//#include "kmeans.h"
#include "kmeans_htd.h"
#include "featureVectors.h"

const int MAX_ELS = 100000;



//------------------------------------------------------
// data: get tiles and extract only tiles of one image
// ExtractImageTiles( "files\\htd_adjt.bin", "files\\htd_adjt_img007.bin", 7 );
//------------------------------------------------------
void ExtractImageTiles( const char* fin, const char* fout, int img_id ) {
  unsigned int i;  
  CDimTileList  tiles;
  CDimTileList  tilesOne;

  std::cout << "Reading the Tiles file\n";
  tiles.fromFile( fin );

  for (i=0; i<tiles.size(); ++i)  
    if ( (tiles[i]->image_id == img_id) && (tiles[i]->position_in_img[2] == 0) ) {
      tilesOne.add( *(tiles[i]) );
    }

  std::cout << "Writing the Tiles file\n";
  tilesOne.toFile( fout );
}

void PrecRec( const char *im_type ) {
  xstring str;
  
  //full=0 clear=90 pure=100
  unsigned int onl_purity = 90;
  unsigned int onl_pure[] = { 0, 90, 100 };

  unsigned int max_iters = 20;
  unsigned int max_clust = 20;
  unsigned int im_id = 27;  
  //char im_type[] = "orig";
  //char im_type[] = "adjt";
  //char im_type[] = "eqlz";
  unsigned int k = 5;
  unsigned int nthreads = 1;
  unsigned int ndims = TEXTURE_DIMS;  
  unsigned int idi=0, ki=2;

  //for (int p=0; p<3; ++p) {
  //  onl_purity = onl_pure[p];

  //exp\\htd_%s_precrec_ONL-%d.txt
  str.sprintf("exp\\htd_%s_precrec_id_ONL-%d.txt", im_type, onl_purity);
  std::ofstream f (str.c_str(), std::ios::out|std::ios::binary);
  if (!f.is_open()) return;

  for (ki=2; ki<max_clust; ++ki) {
  for (idi=0; idi<max_iters; ++idi) {

    k = ki;
    CDimTileList tiles;
    std::cout << "Reading the Tiles file\n";
    str.sprintf("F:\\_textures\\exp\\htd_%s_clust_%.2d_i%.2d.bin", im_type, k, idi);
    //str.sprintf("exp\\htd_%s_img%.3d_clust%.2d_i%.2d.bin", im_type, im_id, k, idi);
    tiles.fromFile( str.c_str() );

    CDimTextureKMeans<texture_type> km( nthreads, k, ndims );
    km.setStdOutProgress();
    
    std::cout << "Classify ONL\n";
    //onl_purity = 100;
    int class_onl = km.getClassClosestToGT( &tiles, RET_LAYER_ONL, onl_purity );
    
    std::cout << "Computing prec/rec for " <<  (unsigned int)tiles.size() << " points...\n" ;
    double precision, recall;
    //onl_purity = 0;
    km.getPrecisionRecall( &tiles, RET_LAYER_ONL, class_onl, onl_purity, precision, recall );
    std::cout << "K: " << k << " ID: " << idi << " Selected: " << class_onl;
    std::cout << " Precision: " << precision << " Recall: " << recall << "\n";

    f << k << " " << idi << " " << precision << " " << recall << "\n";

  } // for idi
  } // for ki
  f.close();
  //} //p
}

void PrecRecOne( const char *im_type, int im_id ) {
  xstring str;
  
  //full=0 clear=90 pure=100
  unsigned int onl_purity = 90;
  unsigned int onl_pure[] = { 0, 90, 100 };

  unsigned int max_iters = 20;
  unsigned int max_clust = 20;
  //unsigned int im_id = 27;  
  //char im_type[] = "orig";
  //char im_type[] = "adjt";
  //char im_type[] = "eqlz";
  unsigned int k = 5;
  unsigned int nthreads = 1;
  unsigned int ndims = TEXTURE_DIMS;  
  unsigned int idi=0, ki=2;

  //for (int p=0; p<3; ++p) {
  //  onl_purity = onl_pure[p];

  //str.sprintf("exp\\htd_%s_img%.3d_clust%.2d_i%.2d.bin", im_type, im_id, k, idi);
  //str.sprintf("exp\\htd_%s_img%.3d_clust_norma%.2d_i%.2d.bin", im_type, im_id, k, idi);
  //str.sprintf("exp\\htd_%s_img%.3d_clust_distn%.2d_i%.2d.bin", im_type, im_id, k, idi);
  //str.sprintf("exp\\htd_%s_img%.3d_clust_norma_dist%.2d_i%.2d.bin", im_type, im_id, k, idi);
  //str.sprintf("exp\\htd_%s_img%.3d_clust_normd_dist%.2d_i%.2d.bin", im_type, im_id, k, idi);


  //exp\\htd_%s_precrec_ONL-%d.txt
  str.sprintf("exp\\htd_%s_precrec_i%.3d_normd_dist_id_ONL-%d.txt", im_type, im_id, onl_purity);
  std::ofstream f (str.c_str(), std::ios::out|std::ios::binary);
  if (!f.is_open()) return;

  for (ki=2; ki<max_clust; ++ki) {
  for (idi=0; idi<max_iters; ++idi) {

    k = ki;
    CDimTileList tiles;
    std::cout << "Reading the Tiles file\n";
    str.sprintf("exp\\htd_%s_img%.3d_clust_normd_dist%.2d_i%.2d.bin", im_type, im_id, k, idi);
    //str.sprintf("exp\\htd_%s_img%.3d_clust%.2d_i%.2d.bin", im_type, im_id, k, idi);
    tiles.fromFile( str.c_str() );

    CDimTextureKMeans<texture_type> km( nthreads, k, ndims );
    km.setStdOutProgress();
    
    std::cout << "Classify ONL\n";
    //onl_purity = 100;
    int class_onl = km.getClassClosestToGT( &tiles, RET_LAYER_ONL, onl_purity );
    
    std::cout << "Computing prec/rec for " <<  (unsigned int)tiles.size() << " points...\n" ;
    double precision, recall;
    //onl_purity = 0;
    km.getPrecisionRecall( &tiles, RET_LAYER_ONL, class_onl, onl_purity, precision, recall );
    std::cout << "K: " << k << " ID: " << idi << " Selected: " << class_onl;
    std::cout << " Precision: " << precision << " Recall: " << recall << "\n";

    f << k << " " << idi << " " << precision << " " << recall << "\n";

  } // for idi
  } // for ki
  f.close();
  //} //p
}

void ClusterOne( const char *im_type, int im_id  ) {

  //int im_id = 27;
  //char im_type[] = "orig";
  //char im_type[] = "adjt";
  //char im_type[] = "eqlz";

  unsigned int max_iters = 1;
  unsigned int max_clust = 13;
  unsigned int ki=5, idi=0;
  unsigned int k = 5;
  unsigned int nthreads = 1;
  unsigned int ndims = TEXTURE_DIMS;
  xstring str;

  // read tiles and get the list of tiles for the first image only
  CDimImageList images;
  std::cout << "Reading the Images file\n";
  images.fromFile("files\\image3d_list.txt");

  CDimTileList  tiles;
  std::cout << "Reading the Tiles file\n";
  //htd_orig_img001.bin
  str.sprintf("files\\htd_%s_img%.3d.bin", im_type, im_id);
  tiles.fromFile( str.c_str() );
  if (tiles.size() <= 1) return;
  std::cout << "Loaded " << (int) tiles.size() << " tiles\n";

  for (ki=2; ki<max_clust; ++ki) {
    for (idi=0; idi<max_iters; ++idi) {
      k = ki;
      CDimTextureKMeans<texture_type> km( nthreads, k, ndims );
      km.setStdOutProgress();
      km.setMaxIterations(300);
      km.run( &images, &tiles );  

      std::cout << "Writing the clustered tiles file\n";
      
      // clustering 60 dims
      //str.sprintf("exp\\htd_%s_img%.3d_clust%.2d_i%.2d.bin", im_type, im_id, k, idi);

      // clustering 60 dims normalized by std
      //str.sprintf("exp\\htd_%s_img%.3d_clust_norma%.2d_i%.2d.bin", im_type, im_id, k, idi);
      
      // clustering 60 dims + 2 normalized dist by max size
      //str.sprintf("exp\\htd_%s_img%.3d_clust_distn%.2d_i%.2d.bin", im_type, im_id, k, idi);
      
      // clustering 60 dims + 2 dist all normalized
      //str.sprintf("exp\\htd_%s_img%.3d_clust_norma_dist%.2d_i%.2d.bin", im_type, im_id, k, idi);
      
      // clustering 60 dims + 2 dist normalized by std
      //str.sprintf("exp\\htd_%s_img%.3d_clust_normd_dist%.2d_i%.2d.bin", im_type, im_id, k, idi);

      // clustering 60 dims using EMD distance
      str.sprintf("exp\\htd_%s_img%.3d_clust_EMDN_%.2d_i%.2d.bin", im_type, im_id, k, idi);
      
      tiles.toFile( str.c_str() );

    } // idi
  } // ki

}

void ClusterAll( const char *im_type ) {

  unsigned int im_id = 1;
  //char im_type[] = "orig";
  //char im_type[] = "adjt";
  //char im_type[] = "eqlz";

  unsigned int max_iters = 20;
  unsigned int max_clust = 20;
  unsigned int ki=5, idi=0;
  unsigned int k = 5;
  unsigned int nthreads = 1;
  unsigned int ndims = TEXTURE_DIMS;
  xstring str;


  // read tiles and get the list of tiles for the first image only
  CDimImageList images;
  std::cout << "Reading the Images file\n";
  images.fromFile("files\\image3d_list.txt");

  CDimTileList  tiles;
  std::cout << "Reading the Tiles file\n";
  //str.sprintf("files\\htd_%s_img%.3d.bin", im_type, im_id);
  str.sprintf("files\\htd_%s2.bin", im_type);
  tiles.fromFile( str.c_str() );

  for (ki=2; ki<max_clust; ++ki) {
    for (idi=0; idi<max_iters; ++idi) {

      k = ki;
      std::cout << "K: " << ki;
      std::cout << " ID: " << idi << "\n";

      CDimTextureKMeans<texture_type> km( nthreads, k, ndims );
      km.setStdOutProgress();
      km.setMaxIterations(300);
      km.run( &images, &tiles );  

      std::cout << "Writing the clustered tiles file\n";
      //str.sprintf("exp\\htd_%s_img%.3d_clust%.2d_i%.2d.bin", im_type, im_id, k, idi);
      str.sprintf("F:\\_textures\\exp\\htd_%s_clust_dist%.2d_i%.2d.bin", im_type, k, idi);
      tiles.toFile( str.c_str() );

    } // idi
  } // ki

}

int main(int argc, char argv[])
{
  //ExtractImageTiles( "files\\htd_orig2.bin", "files\\htd_orig_img001.bin", 1 ); 
  //ClusterOne( "orig", 1 );
  //PrecRecOne( "orig", 1 );

  //ClusterOne( "orig", 7 );
  //PrecRecOne( "orig", 7 );

  ClusterOne( "orig", 27 );
  //PrecRecOne( "orig", 27 );

  //ClusterAll( "orig" );
  //ClusterAll( "adjt" );
  //ClusterAll( "eqlz" );
  
  //PrecRec( "orig" );
  //PrecRec( "adjt" );
  //PrecRec( "eqlz" );

  return 0;

  /*
  // read fixed ASCII testures and update the texture files
  CDimTileList  tiles;
  CDimTileListTxt il;
  CDimTileDescriptorTxt tdT;

  std::cout << "Reading the Tiles file\n";
  tiles.fromFile( "files\\htd_orig.bin" );

  il.fromFile( "files\\HTD_original.dat", "files\\tile_metadata.ifl" );
  std::cout << "Red the ASCII file\n";

  unsigned int i;
  for (i=0; i<il.size(); ++i) {
    memcpy(tiles[i]->texture, il[i]->texture, sizeof(float)*60 );
  }

  std::cout << "Writing the Tiles file\n";
  tiles.toFile( "files\\htd_orig2.bin" );


  /*
  unsigned int i;  
  CDimTileList  tiles;
  CDimImageList images;
  CDimTileList  tilesOne;

  std::cout << "Reading the Images file\n";
  images.fromFile("files\\image_list.txt");

  std::cout << "Reading the Tiles file\n";
  tiles.fromFile( "files\\htd_orig.bin" );

  int maxcol = tiles.getMaxPosInCol(7);
  int maxrow = tiles.getMaxPosInRow(7);
  unsigned int ji = tiles.getTileIndex(7, 0, 0, 0);
  unsigned int je = tiles.getTileIndex(7, maxcol, maxrow, 0);

  for (i=0; i<tiles.size(); ++i)  
    if ( (tiles[i]->image_id == 7) && (tiles[i]->position_in_img[2] == 0) ) {
      CDimTileDescriptor d = *(tiles[i]);
      tilesOne.add( *(tiles[i]) );
    }

  std::cout << "Writing the Tiles file\n";
  tilesOne.toFile( "files\\htd_orig_img007.bin" );

  return 0;

  /*
  unsigned int ki=5, idi=0;
  unsigned int k = 5;
  unsigned int nthreads = 1;
  unsigned int ndims = TEXTURE_DIMS;

  //for (ki=2; ki<21; ++ki) {
  //  for (idi=0; idi<3; ++idi) {

      k = ki;
      // read tiles and get the list of tiles for the first image only
      CDimImageList images;
      std::cout << "Reading the Images file\n";
      images.fromFile("files\\image3d_list.txt");

      CDimTileList  tiles;
      std::cout << "Reading the Tiles file\n";
      tiles.fromFile( "files\\htd_orig_img007.bin" );

      CDimTextureKMeans<texture_type> km( nthreads, k, ndims );
      km.setStdOutProgress();
      km.setMaxIterations(2000);
      km.run( &images, &tiles );  

      std::cout << "Writing the clustered tiles file\n";
      xstring str;
      str.sprintf("files\\htd_orig_img007_clust_dist%.2d_i%.2d.bin", k, idi);
      tiles.toFile( str.c_str() );

  //  } // idi
  //} // ki

  return 0;

  /*
  unsigned int ndims = 3;
  unsigned int k = 20;
  unsigned int nthreads = 2;

  // first test the 2D case
  CDimVectorOfFeatures<texture_type> vf(ndims);

  unsigned int num_els = k*5000;
  std::cout << "Loading data\n";
  vf.reserve(num_els);
  vf.fromFile("matlab\\data1.dat");

  CDimThreadedKMeansBase<texture_type> kmb( nthreads, k, ndims );
  kmb.setStdOutProgress();
  kmb.setMaxIterations(2000);
  kmb.run( &vf );
  kmb.centers()->toFile("matlab\\centroids_final.dat");
  kmb.labs()->toFile("matlab\\labels_final.dat");

  return 0;


  /*
  CDimTileList tiles;
  CDimImageList images;

  images.fromFile("files\\image3d_list.txt");
  std::cout << "Red the Images file\n";

  tiles.fromFile( "files\\htd_orig.bin" );
  std::cout << "Red the Tiles file\n";


  //CDimTextureKMeans<textu_real> km( 1, 2, TEXTURE_DIMS );
  //km.run( &images, &tiles );  
  
  CDimTextureKMeans<textu_real> *km; 
  km = new CDimTextureKMeans<textu_real>( 1, 2, TEXTURE_DIMS );
*/

}

//------------------------------------------------------
// example: cluster 3D points
//------------------------------------------------------
/*
  unsigned int ndims = 3;
  unsigned int k = 20;
  unsigned int nthreads = 2;

  // first test the 2D case
  CDimVectorOfFeatures<texture_type> vf(ndims);

  unsigned int num_els = k*5000;
  std::cout << "Loading data\n";
  vf.reserve(num_els);
  vf.fromFile("matlab\\data1.dat");

  CDimThreadedKMeansBase<texture_type> kmb( nthreads, k, ndims );
  kmb.setStdOutProgress();
  kmb.setMaxIterations(2000);
  kmb.run( &vf );
  kmb.centers()->toFile("matlab\\centroids_final.dat");
  kmb.labs()->toFile("matlab\\labels_final.dat");
*/



/*
 
 // read tiles, process and create list
  CDimMapList map;
  map.fromFile("files\\map_list.txt");

  CDimTileList ol;
  CDimTileListTxt il;
  CDimTileDescriptor td;

  il.fromFile( "files\\HTD_original.dat", "files\\tile_metadata.ifl" );
  std::cout << "Red the ASCII file\n";

  unsigned int i;
  for (i=0; i<il.size(); ++i) {
    td = *il[i];
    td.image_id = map[td.image_id-1]->image_5d_id;
    td.position_in_img[1] = il[i]->tile_pos_rowcol[0]*il[i]->tile_step;
    td.position_in_img[0] = il[i]->tile_pos_rowcol[1]*il[i]->tile_step;
    ol.add( td );
  }

  ol.toFile("files\\htd_orig.bin");
*/


/*
// read original images and create lists
  CDimImageList old_il, new_il;
  old_il.fromFile("files\\image_list.txt");
  CDimMapList map;

  unsigned int i;
  int n=-1;
  CDimImageItem ti, ii;
  CDimMapItem mi;
  for (i=0; i<old_il.size(); ++i) {
    mi.image_plane_id = i+1;
    mi.image_5d_id = new_il.size()+1;
    ii = *old_il[i];

    if ( (old_il[i]->image_size[2] == 0) && (n != -1) ) {
      ti.image_size[2] = ti.image_size[2]+1;
      ti.image_id = new_il.size()+1;
      mi.image_5d_id = ti.image_id+1;
      new_il.add(ti);
    }

    if ( old_il[i]->pixel_size[2] == 0.0 ) {
      ii.image_id = new_il.size()+1;
      ii.image_size[2] = 1;
      mi.image_5d_id = ii.image_id;
      new_il.add(ii);
      n=-1;
    }
    else {
      if (old_il[i]->image_size[2] == 0) n = 0;
      ti = *old_il[i];
      ++n;
    }
    map.add(mi);
  }

  if (n != -1) {
    ti.image_size[2] = ti.image_size[2]+1;
    ti.image_id = new_il.size()+1;
    map.back()->image_5d_id = ti.image_id;
    new_il.add(ti);
  }
  
  new_il.toFile("files\\image3d_list.txt");
  map.toFile("files\\map_list.txt");
*/



