/*****************************************************************************
 descriptors for textures, tiles and images - ASCII format

 DEFINITION

 Author: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>
 Copyright (c) 2006 Vision Research Lab, UCSB <http://vision.ece.ucsb.edu>

 History:
   04/19/2006 16:20 - First creation

 Ver : 1
*****************************************************************************/

#ifndef DIM_TEXTURE_DESCRIPTORS_TXT
#define DIM_TEXTURE_DESCRIPTORS_TXT

#include "textureDescriptors.h"


//----------------------------------------------------------------------------
// tiles
//----------------------------------------------------------------------------

class CDimTileDescriptorTxt: public CDimTileDescriptor 
{
public:
  // ascii data
  unsigned int  tile_pos_rowcol[2]; // tile's row column position
  std::string   image_name;         // image file name
  unsigned int  image_size[2];      // image size WH
  unsigned int  tile_step;          // offset for each new tile

  // IO
  int fromFile(std::ifstream *fs) { fs; return -1; };
  int fromFile(std::ifstream *fs1, std::ifstream *fs2);

  CDimTileDescriptorTxt();
};

class CDimImageItemTxt: public CDimImageItem 
{
public:
  int fromFile(std::ifstream *fs);
};

//----------------------------------------------------------------------------
// ground truth
//----------------------------------------------------------------------------

class CDimGTDescriptorTxt: public CDimTileDescriptor 
{
public:
  // layer data: 1-BG 2-ILM 3-GCL 4-IPL 5-INL 6-OPL 7-ONL 8-IS 9-OS 10-RPE 11-M/L
  int  layers_percents[11]; // percent of each layer in the tile
  unsigned int  tile_pos_rowcol[2]; // tile's row column position

  // IO
  int fromFile(std::ifstream *fs);

  CDimGTDescriptorTxt(): CDimTileDescriptor() {};
};

class CDimGTTileListTxt: public CDimItemListBase <CDimGTDescriptorTxt>
{
};

//----------------------------------------------------------------------------
// lists
//----------------------------------------------------------------------------

class CDimTileListTxt: public CDimItemListBase <CDimTileDescriptorTxt>
{
public:
  int fromFile(const char *fileName1, const char *fileName2);
  int fromFile(const std::string *fileName1, const std::string *fileName2);

  // support
  int findImageById(unsigned int imID);
  int findMaxZTileByImageId(unsigned int imID);
protected:
  int fromFile(const char *fileName) { fileName; return 0; };
  int fromFile(const std::string *fileName) { fileName; return 0; };
};

class CDimImageListTxt: public CDimItemListBase <CDimImageItemTxt> { };


#endif // DIM_TEXTURE_DESCRIPTORS_TXT
